//
// geticon.js
// Copyright (c) 2009 Kinesys Ltd
//
//
// The following set of functions enable icons to be extracted from PE (Portable Executable) files (exe's, dll's etc)
//
// Some of the icon extraction functionality in this script was developed using information from a VBScript package and
// documentation written by Joe Priestley (www.jsware.net)
//
// Usage:
//
// saveIcons(exePath, savePath) - This function will extract all icon images from the specified PE file and save them as
//   icon files into the specified folder (as iconxx.ico).
//
// saveIconGrps(exePath, savePath) - Ths function will extract all icon image groups (icons with multiple images) from the
//   specified PE file and save them as icon files into the specified folder (as iconxx.ico).
//
// saveIconGroup(iconGrpNum, savePath) - This function extracts a specified numbered icon group from the PE file and saves
//   it as the specified icon file name.
//   The getRsrc(exePath) function (which extracts the resource section of the PE file) needs to be called before using this
//   function (allows you to then make multiple calls to saveIconGroup without recalling getRsrc)
//
// The following error codes may be returned:
//
//  0 Function succeeded
// -1 File does not exist
// -2 No resource table header
// -3 No icons in resource table
// -4 No PE signature found
// -5 No icon for this iconNum or grpNum
//

var fso = new ActiveXObject("Scripting.FileSystemObject");

var rsrcLocation; // pointer to rsrc section
var rsrcSize; // size of rsrc section
var rsrc; // rsrc section
var rsrcVOffset; // rsrc virtual offset
var namIconCount, numIconCount, iconCount, iconTableOffset;
var namGrpIconCount, numGrpIconCount, iconGrpCount, iconGrpTableOffset;

var iconHdrs; // hdrs for each icon
var iconData; // image data for each icon
var iconIds; // iconIds[id] = icontable index
var grpHdrs; // hdrs for each icon group

// get resource section of PE file
function getRsrc(exePath){
	var f, first2K, ptr, namRsrcCount, numRsrcCount, s, readPtr, x, offset, hdr;
	var iconDataLoc, iconDataSize;
	var hasIcons = false;
	var hasIconGrps = false;

	if(!fso.FileExists(exePath)){
		return -1; // File doesn't exist
	}
	f = fso.OpenTextFile(exePath,1);
	first2K = f.Read(2048);
	f.Close();
	ptr = getNumFromStr(first2K.substr(60,2));
	if(first2K.substr(ptr,4) != "PE" + String.fromCharCode(0,0)){
		return -4; // No PE signature found
	}
	ptr = first2K.search(/\.rsrc/);
	if(ptr == 0){
		return -2; // No resource table header
	}
	ptr += 12;
	rsrcVOffset = getNumFromStr(first2K.substr(ptr,4));
	rsrcSize = getNumFromStr(first2K.substr(ptr + 4,4));
	rsrcLocation = getNumFromStr(first2K.substr(ptr + 8,4));
	ptr = first2K.search(/\.aspack/);
	if(ptr > 0){
		ptr += 12;
		rsrcVOffset += getNumFromStr(first2K.substr(ptr,4));
		rsrcLocation += getNumFromStr(first2K.substr(ptr + 8,4));
	}
	f = fso.OpenTextFile(exePath,1);
	f.Skip(rsrcLocation);
	rsrc = f.Read(rsrcSize);
	f.Close();
	namRsrcCount = getNumFromStr(rsrc.substr(12,2));
	numRsrcCount = getNumFromStr(rsrc.substr(14,2));
	if(numRsrcCount == 0){
		f.Close();
		return -3; // No icons in resource table
	}
	ptr = 16;
	if(namRsrcCount > 0){
		ptr += namRsrcCount * 8;
	}
	for(x = 1; x <= numRsrcCount; x++){
		s = rsrc.substr(ptr,8);
		if(s.charCodeAt(0) == 3){
			hasIcons = true;
			iconTableOffset = getNumFromStr(s.substr(4,3));
		}
		if(s.charCodeAt(0) == 14){
			hasIconGrps = true;
			iconGrpTableOffset = getNumFromStr(s.substr(4,3));
		}
		ptr += 8;
	}
	if(!hasIcons){
		f.Close();
		return -3; // No icons in resource table
	}
	namIconCount = getNumFromStr(rsrc.substr(iconTableOffset + 12, 2));
	numIconCount = getNumFromStr(rsrc.substr(iconTableOffset + 14, 2));
	iconTableOffset += 16;
	iconCount = namIconCount + numIconCount;
	namGrpIconCount = getNumFromStr(rsrc.substr(iconGrpTableOffset + 12, 2));
	numGrpIconCount = getNumFromStr(rsrc.substr(iconGrpTableOffset + 14, 2));
	iconGrpTableOffset += 16;
	iconGrpCount = namGrpIconCount + numGrpIconCount;
	return 0;
}

// get icon image data from icon id
function getIconFromId(iId){
	var offset, s, x, iconDataLoc, iconDataSize, iData = "";
	
	offset = iconTableOffset + iconIds[iId] * 8;
	s = rsrc.substr(offset,8);
	for(x = 1; x <= 8; x++){
		if(offset == 0 && s.charCodeAt(7) == 0){
			return ""; // no icon for this iId
		}
		if(s.charCodeAt(7) > 127){
			offset = getNumFromStr(s.substr(4,3)) + 16;
			s = rsrc.substr(offset,8);
		}else{
			break;
		}
	}
	offset = getNumFromStr(s.substr(4,3));
	s = rsrc.substr(offset,8);
	if(offset > 0){
		iconDataLoc = getNumFromStr(s.substr(0,3)) - rsrcVOffset;
		iconDataSize = getNumFromStr(s.substr(4,3));
		iData = rsrc.substr(iconDataLoc,iconDataSize);
	}
	return iData;
}

// get all icon ids from icontable and assign numerical sequence no.
function getIconIds(){
	var iIds = new Array();
	var iconNum, offset;

	offset = iconTableOffset;
	for(iconNum = 0; iconNum < iconCount; iconNum++){
		iIds[getNumFromStr(rsrc.substr(offset,2))] = iconNum;
		offset += 8;
	}
	iconIds = iIds.slice();
}

// get all icon groups from resource section
function getIconGrps(){
	var grpNum, offset, x, s, iconGrpHdrLoc, iconGrpHdrSize, grpIconCount, grpIconNum, iconNum, iHdr, iId;
	var gHdrs = new Array();
	var iHdrs = new Array();
	var iData = new Array();
	for(grpNum = 0; grpNum < iconGrpCount; grpNum++){
		offset = iconGrpTableOffset + grpNum * 8;
		s = rsrc.substr(offset,8);
		for(x = 1; x <= 8; x++){
			if(offset == 0 && s.charCodeAt(7) == 0){
				return -5; // no icon for this iconNum
			}
			if(s.charCodeAt(7) > 127){
				offset = getNumFromStr(s.substr(4,3)) + 16;
				s = rsrc.substr(offset,8);
			}else{
				break;
			}
		}
		offset = getNumFromStr(s.substr(4,3));
		s = rsrc.substr(offset,8);
		if(offset > 0){
			iconGrpHdrLoc = getNumFromStr(s.substr(0,3)) - rsrcVOffset;
			iconGrpHdrSize = getNumFromStr(s.substr(4,3));
			gHdrs[grpNum] = rsrc.substr(iconGrpHdrLoc,iconGrpHdrSize);
			grpIconCount = getNumFromStr(gHdrs[grpNum].substr(4,2));
			for(grpIconNum = 0; grpIconNum < grpIconCount; grpIconNum++){
				iHdr = gHdrs[grpNum].substr(6 + grpIconNum * 14,14);
				iId = getNumFromStr(iHdr.substr(12,2));
				iconNum = iconIds[iId];
				iHdr = iHdr.substr(0,12);
				iHdrs[iconNum] = iHdr;
				iData[iconNum] = getIconFromId(iId);
			}
		}
	}
	grpHdrs = gHdrs.slice();
	iconHdrs = iHdrs.slice();
	iconData = iData.slice();
}

// get all separate icon images from a PE file and save to icon files
function saveIcons(exePath, savePath){
	var err, iconNum, icon;
	
	err = getRsrc(exePath);
	if( err < 0){
		return err;
	}
	getIconIds();
	getIconGrps();
	for(iconNum = 0; iconNum < iconCount; iconNum++){
		if(iconHdrs[iconNum] != ""){
			icon = String.fromCharCode(0,0,1,0,1,0) + iconHdrs[iconNum] + String.fromCharCode(22, 0, 0, 0) + iconData[iconNum];
			f = fso.CreateTextFile(savePath + "icon" + (iconNum + 1) + ".ico", true);
			f.Write(icon);
			f.Close();
		}
	}
}

// get all group icons from a PE file and save to icon files
function saveIconGrps(exePath, savePath){
	var err, grpNum, fileHdr, GrpIconNum, iId, iconNum, iHdr, iDataOffset, iData;
	
	err = getRsrc(exePath);
	if( err < 0){
		return err;
	}
	getIconIds();
	getIconGrps();
	for(grpNum = 0; grpNum < iconGrpCount; grpNum++){
		iData = ""
		if(grpHdrs[grpNum] != ""){
			f = fso.CreateTextFile(savePath + "icongrp" + grpNum + ".ico", true);
			fileHdr = grpHdrs[grpNum].substr(0,6);
			grpIconCount = getNumFromStr(fileHdr.substr(4,2));
			f.Write(fileHdr);
			iDataOffset = 6 + 16 * grpIconCount;
			for(grpIconNum = 0; grpIconNum < grpIconCount; grpIconNum++){
				iHdr = grpHdrs[grpNum].substr(6 + grpIconNum * 14);
				iId = getNumFromStr(iHdr.substr(12,2));
				iconNum = iconIds[iId];
				iHdr = iHdr.substr(0,12);
				iData += iconData[iconNum];
				iHdr += getPntrFromNum(iDataOffset);
				f.Write(iHdr);
				iDataOffset += iconData[iconNum].length;
			}
			f.Write(iData);
			f.Close();
		}
	}
}

// get numbered group icon from a PE file and save to icon file
// - assumes getRsrc() has been previously called
function saveIconGroup(iconGrpNum, savePath){
	var err, offset, s, x, iconGrpHdrLoc, iconGrpHdrSize, iconGrpHdr, grpIconCount, grpIconNum, iHdr, iId;
	var f, fileHdr, iDataOffset, iconData, grpIconData = "";
	
	err = getIconIds();
	if( err < 0){
		return err;
	}
	offset = iconGrpTableOffset + iconGrpNum * 8;
	s = rsrc.substr(offset,8);
	for(x = 1; x <= 8; x++){
		if(offset == 0 && s.charCodeAt(7) == 0){
			return -5; // no icons for this grpNum
		}
		if(s.charCodeAt(7) > 127){
			offset = getNumFromStr(s.substr(4,3)) + 16;
			s = rsrc.substr(offset,8);
		}else{
			break;
		}
	}
	offset = getNumFromStr(s.substr(4,3));
	s = rsrc.substr(offset,8);
	if(offset > 0){
		iconGrpHdrLoc = getNumFromStr(s.substr(0,3)) - rsrcVOffset;
		iconGrpHdrSize = getNumFromStr(s.substr(4,3));
		iconGrpHdr = rsrc.substr(iconGrpHdrLoc,iconGrpHdrSize);
		grpIconCount = getNumFromStr(iconGrpHdr.substr(4,2));

		f = fso.CreateTextFile(savePath, true);
		fileHdr = iconGrpHdr.substr(0,6);
		f.Write(fileHdr);
		iDataOffset = 6 + 16 * grpIconCount;
		for(grpIconNum = 0; grpIconNum < grpIconCount; grpIconNum++){
			iHdr = iconGrpHdr.substr(6 + grpIconNum * 14);
			iId = getNumFromStr(iHdr.substr(12,2));
			iHdr = iHdr.substr(0,12);
			iconData = getIconFromId(iId);
			grpIconData += iconData;
			iHdr += getPntrFromNum(iDataOffset);
			f.Write(iHdr);
			iDataOffset += iconData.length;
		}
		f.Write(grpIconData);
		f.Close();
		return 0;
	}
	return -5; // no icon for this grpNum
}

//
// Javascript doesn't really have any built-in functionality for dealing with binary files, and so they have to be accessed
// as text files.  Also Javascript deals with strings exclusively in Unicode.  This means that byte data read from a file
// into a string is internally converted to Unicode.  This isn't a problem if writing the strings back directly to a file
// as text, as it is converted back to standard ASCII single byte text.  But, if you want to manipulate the byte data by
// using charCodeAt() and fromCharCode(), then you need to take into account the Unicode substitutions for certain chars.
//
// For instance, if you read a byte from a file that was code 128, into a string, that char will be represented internally
// as Unicode \u20ac.  If you use charCodeAt() to get it's code, you will get 8364 (0x20ac).  If you try to create a char
// using code 128 with fromCharCode(), it will error.  You have to use 8364 (0x20ac) which would then be converted to byte
// code 128 when written to file.
//
// The following creates a string of the 256 possible characters with appropriate Unicode substitutions, that can then be
// used to convert to and from the byte code values.
//

// bytes.txt is an ASCII file consisting of the 256 byte codes
var f = fso.OpenTextFile(gadgetPath + "\\templates\\bytes.txt", 1);
// chars is a string that will contain the equivalent unicode values
var chars = f.ReadAll();
f.Close();
var factors = new Array(1,256,65536,16777216);

// Get byte code from a char (while taking care of Unicode substitutions)
function getCodeFromChar(c){
	return chars.indexOf(c);
}

// Get char from byte code  (while taking care of Unicode substitutions)
function getCharFromCode(n){
	return chars.substr(n,1);
}

// Converts a 1 to 4 byte string (read from a binary file) to an integer (assumes reverse byte order and takes care of
// Unicode substitutions)
function getNumFromStr(str){
	var x, num = 0;
	if(str.length > 4){
		str = str.substr(0,4);
	}
	for(x = 0; x < str.length; x++){
		num += getCodeFromChar(str.substr(x,1)) * factors[x];
	}
	return num;
}

// Converts a number into a 4 byte string representing a binary integer when written to file (assumes reverse byte order
// and takes care of Unicode substitutions)
function getPntrFromNum(num){
	var x, y, s = "";
	for(x = 3; x >= 0; x--){
		y = Math.floor(num/factors[x]);
		num = num - y * factors[x];
		s = "" + getCharFromCode(y) + s;
	}
	return s;
}
